class List{
    // Costanti
    private static final int KIND_NIL=0;
    private static final int KIND_CONS=1;

    // Campi della classe
    private int kind;
    private int head;
    private List tail;

    // Costruttore
    private List(int kind, int head,List tail){
        this.kind=kind; this.head=head; this.tail=tail;
    }

    // Metodi di factory
    public static List Nil(){ return new List(KIND_NIL,-1,null);}
    public static List Cons(int head, List tail){
        return new List(KIND_CONS,head,tail);
    }

    // Funzionalit dell'ADT
    public static boolean isNil(List l){ return (l.kind==KIND_NIL);}

    public static int getHead(List l){ return l.head; }

    public static List getTail(List l){
        return l.kind==KIND_NIL?null:l.tail;
    }

    public static boolean isEqual(List l1,List l2){
        if (l1.kind==KIND_NIL && l2.kind==KIND_NIL)
            return true;
        if (l1.kind!=KIND_NIL ^ l2.kind!=KIND_NIL) return false;
        return (l1.head==l2.head) && (isEqual(l1.tail,l2.tail));
    }
    
    public static List eraseAll(List l,int i) {
        if (l.kind==KIND_NIL) return Nil();
        if (l.head==i) return eraseAll(l.tail, i);
        return Cons(l.head,eraseAll(l.tail,i));
    }
    
    static public boolean search(List l,int i){
        if (l.kind==KIND_NIL) return false;
        if (l.head==i) return true;
        return search(l.tail,i);
    }
    
    static public int count(List l,int i){
        if (l.kind==KIND_NIL) return 0;
        if (l.head==i) return count(l.tail,i)+1;
        return count(l.tail,i);
    }
    
    public static List append(List l,int i) {
        if (l.kind==KIND_NIL)
            return Cons(i, Nil());
        return Cons(l.head,append(l.tail,i));
    }
    
    public static List concat(List l1,List l2) {
        if (l1.kind==KIND_NIL) return l2;
        return Cons(l1.head,concat(l1.tail,l2));
    }
    
    public static List insertodds(List l,int i){
        if (l.kind==KIND_NIL) return Cons(i,Nil());
        return Cons(i, Cons(l.head,insertodds(l.tail,i)));
    }
    
    public static List fromArray(int[] a){
        List l=Nil();
        for (int i=a.length-1;i>=0;i--)
            l=List.Cons(a[i],l);
        return l;
    }

    public static String toString(List l){
        if (l.kind==KIND_NIL) return "";
        return ""+l.head+";"+toString(l.tail);
    }
    
    public static List zip(List l1,List l2){
    	if (l1.kind==KIND_NIL) return l2;
    	if (l2.kind==KIND_NIL) return l1;
    	return Cons(l1.head,Cons(l2.head,zip(l1.tail,l2.tail)));
    }
    
    public static List unzipodds(List l){
    	if (l.kind==KIND_NIL) return Nil();
    	if (l.tail.kind==KIND_NIL) return Cons(l.head,Nil());
    	return Cons(l.head,unzipodds(l.tail.tail));
    }
    
    public static List unzipevens(List l){
    	if ((l.kind==KIND_NIL)||(l.tail.kind==KIND_NIL)) return Nil();
    	return Cons(l.tail.head,unzipevens(l.tail.tail));
    }
    	
    	
}
