/**
 * Questa classe implementa un semplice tipo di lista, con elementi di tipo intero.
 * Una lista e' una collezione lineare di elementi.
 *
 * La classe fornisce metodi per aggiungere un elemento in coda (metodo <code>addElement</code>),
 * per svuotare la lista (metodo <code>empty</code>), per leggere l'elemento data
 * la sua posizione o indice (metodo <code>getElement</code>), per conoscere la
 * lunghezza corrente della lista (metodo <code>getLength</code>), per testare se la lista
 * sia vuota (metodo <code>isEmpty</code>).
 *
 * La strategia implementativa utilizzata e' caratterizzata dall'uso
 * di un array <code>elements</code> (tra i dati privati della classe), ove
 * vengono memorizzati di volta in volta gli elementi inseriti.
 * Per tener traccia del numero di elementi correnti si utilizza
 * una variabile intera <code>nElements</code>, sempre mantenuta
 * come campo privato della classe.  L'array viene creato si una
 * lunghezza prefissata (10 elementi) e viene espanso mano a mano
 * ne sorge l'esigenza, ovvero: quando l'inserimento di un elemento
 * comporta l'overflow dell'array, se ne costruisce uno nuovo
 * di una lunghezza superiore (di 10 elementi) e vi si ricopiano
 * tutti gli elementi del precedente.
 *
 *
 *
 * @author Alessandro Ricci
 * @version 1.0
 *
 */
public class ACMEList {

    // lunghezza corrente della lista
    private int nElements;

    // array ove sono contenuti gli elementi della lista
    private int[] elements;


    /**
     * Costruisce una lista, inizialmente vuota
     */
    public ACMEList(){
        // creiamo un array iniziale atto a contenere 10 elementi
        elements = new int[10];
        // il numero di elementi a lista creata e' zero
        nElements = 0;
    }

    /**
     * Aggiunge un elemento in coda a questa lista
     *
     * @param element elemento da aggiungere
     */
    public void addElement(int element){

        // aggiungiamo l'elemento in coda alla lista
        // ovvero nella posizione mantenuta dalla variabile nElements
        elements[nElements]=element;
        // incrementiamo il numero corrente di elementi presenti
        nElements++;

        // nel caso in cui il numero di elementi
        // sia superiore alla lunghezza attuale dell'array
        // espandiamo l'array
        if (nElements >= elements.length){
            expand(10);
        }
    }

    /**
     * Restituisce l'elemento di posizione specificata
     *
     * @param index  indice dell'elemento da restituire
     * @return  elemento della lista
     */
    public int getElementAt(int index){
        return elements[index];
    }

    /**
      * Elimina tutti gli elementi di questa lista, rendendola vuota.
      */
     public void empty(){
         nElements=0;
     }

    /**
     * Restituisce il numero degli elementi di questa lista (la sua lunghezza)
     *
     * @return numero elementi della lista
     */
    public int getLength(){
        return nElements;
    }

    /**
     * Testa se la lista e' vuota
     *
     * @return true se la lista e' vuota, ovvero contiene 0 elementi
     */
    public boolean isEmpty(){
        return nElements == 0;
    }


    /**
     * Espande l'array interno utilizzato per mantenere gli elementi della lista
     *
     * @param nElements numero di elementi da aggiungere
     */
    private void expand(int nElements){
        // creiamo un array di n elementi piu' lungo rispetto al precedente
        int[] newElements = new int[elements.length + nElements];
        // copiamo gli elementi dal vecchio al nuovo array
        for (int i=0; i < elements.length; i++){
            newElements[i]= elements[i];
        }
        // sostituiamo il vecchio array con quello nuovo
        elements = newElements;
    }

}
