import java.util.Vector;

/**
 *Questa classe crea un oggetto di tipo Libretto in grado di rappresentare il 
 *libretto universitario di uno studente.
 *
 *La classe fornisce metodi per registrare un esame (<code>registraEsame</code>),
 *per trovare il voto di un esame gi registrato(<code>cercaVoto</code>), per 
 *calcolare la media dei voti(<code>calcolaMedia</code>) e per visualizzare su 
 *Standard output il contenuto di tutto il libretto (<code>visualizza</code>).
 *
 *Questo oggetto contiene 4 campi di tipo String (indicanti: nome del corso di 
 *studio, nome e cognome studente e la sua matricola), uno di tipo int (indicante 
 *l'anno di immatrcolazione) e un riferimento a un altro oggetto della classe 
 *<code>Vector</code> fornita dalla Sun.
 *Nell'oggetto <code>Vector</code> vanno memorizzati gli esami rappresentati da 
 *oggetti di tipo <code>Exam</code> contenenti informazioni sul nome dell'esame,
 *il voto e la data di svolgimento, memorizzata come oggetto di tipo <code>ACMEDate</code>.
 *   
 * @author Andrea Urbini
 */

public class Libretto{
	
	//Campi privati
	
	private String nomeCorso,nome,cognome,matricola; 
	
	private int annoIscrizione;
	
	private Vector listaEsami;
	
	/** 
	 *Costruisce un libretto con i dati forniti in ingresso e con la 
	 *lista degli esami vuota.
	 *
	 * @param nomeCorso nome del corso di studi frequentato
	 * @param nome nome dello studente
	 * @param cognome cognome dello studente
	 * @param matricola numero di matricola dello studente
	 * @param annoIscrizione anno di iscrizione al corso di studio
	 */
	
	public Libretto(String nomeCorso,String nome,String cognome,String matricola, int annoIscrizione){
		this.nomeCorso=nomeCorso;
		this.nome=nome;
		this.cognome=cognome;
		this.matricola=matricola;
		this.annoIscrizione=annoIscrizione;
		listaEsami=new Vector();
	} 
	
	/**
	 *Registra l'esame sul libretto
	 *
	 * @param data data di svolgimento dell'esame
	 * @param nomeEsame nome della materia dell'esame
	 * @param voto voto in trentesimi
	 */
	
	public void registraEsame(ACMEDate data,String nomeEsame,int voto){
		listaEsami.add(new Exam(data,nomeEsame,voto));
	}
	
	/**
	 *Ricerca il voto dell'esame specificato
	 *
	 * @param nomeEsame nome dell'esame
	 * @return -1 se l'esame non  presente nel libretto 
	 */ 
	
	public int cercaVoto(String nomeEsame){
		for (int i=0;i<listaEsami.size();i++){
			Exam esame=(Exam)listaEsami.elementAt(i);
			if (nomeEsame==esame.getName()){
				return esame.getMarks();
			}
		}
		return -1;
	}
	
	/**
	 *Calcola la media dei voti degli esami registrati nel libretto
	 *
	 * @return media dei voti
	 */
	 
	public double calcolaMedia(){
		double media=0;
		for (int i=0;i<listaEsami.size();i++){
			Exam esame=(Exam)listaEsami.elementAt(i);
			media+=esame.getMarks();
		}
		return media/listaEsami.size();
	}
	
	/**
	 *Stampa su standard output il contenuto del libretto
	 */
	
	public void visualizza(){
		System.out.println("\n"+"Libretto universitario"+"\n");
		System.out.println("Nome: "+nome);
		System.out.println("Cognome: "+cognome);
		System.out.println("Matricola: "+matricola);
		System.out.println("Corso di laurea: "+nomeCorso);
		System.out.println("Anno di iscrizione: "+annoIscrizione);
		System.out.println("\n"+"Esami sostenuti"+"\n");
		for (int i=0;i<listaEsami.size();i++){
			Exam esame=(Exam)listaEsami.elementAt(i);
			System.out.println(""+esame.toString());
		}
	}

}		