/**
 * Questa classe estende la classe Plotter aggiungendo nuovi metodi per disegnare
 * segmenti (drawLine), per disegnare poligoni (drawPolygon) e per disegnare 
 * circonferenze (drawCircle).
 * 
 * @author Andrea Urbini
 */


public class ACMEPlotter extends Plotter{
	
	/**
	 * Costruisce un oggetto ACMEPlotter
	 */
	
	public ACMEPlotter(){
		super();
	}
	
	/**
	 * Disegna un segmento tra i due punti ingresso. Alla fine del tracciamento
	 * il pennino si trova nel secondo punto.
	 * @param p1 primo punto
	 * @param p2 secondo punto
	 */
	
	public void drawLine(Point p1,Point p2){
		up();
		moveTo(p1);
		down();
		moveTo(p2);
	}
	
	/**
	 * Disegna un cerchio. Alla fine del tracciamento il pennino si trova sul
	 * centro della circonferenza. .
	 * @param centro centro del cerchio
	 * @param r raggio del cerchio
	 */
	
	public void drawCircle(Point centro,double r){
		//Calcola le coordinate del centro
		double x0=centro.getX();
		double y0=centro.getY();
		//Sposta il pennino nel punto corrispondente ad alfa=0
		up();
		Point start=new Point(x0+r*Math.cos(0),y0+r*Math.sin(0));
		moveTo(start);
		down();
		//Calcola i punti al variare di alfa (precisione 0.01) e li collega 
		for (double alfa=0;alfa<=2*Math.PI;alfa+=0.01){
			Point punto=new Point(x0+r*Math.cos(alfa),y0+r*Math.sin(alfa));
			moveTo(punto);
		}
		//Colleaga l'ultimo punto con il primo.
		moveTo(start);
		//Sposta il pennino al centro
		up();
		moveTo(centro);
	}
	
	/**
	 * Disegna un poligono
	 * @param vertici array contenete i vertici del poligono
	 */
	 
	public void drawPolygon(Point[] vertici){
		//Sposta il pennino al primo vertice del poligono
		up();
		moveTo(vertici[0]);
		down();
		//Collega tutti i vertici
		for (int i=1;i<vertici.length;i++){
			moveTo(vertici[i]);
		}
		//Collega l'ultimo vertice con il primo.
		moveTo(vertici[0]);
	} 
}
		