/**
 * Questa classe rappresenta l'astrazione di un cerchio caratterizzato da un 
 * colore e uno spessore del pennino che lo disegna. Inoltre vanno definiti 
 * anche il centro  e il raggio. 
 *
 * @author Andrea Urbini
 */  


public class Cerchio extends Figura{
	
	private double raggio;
	private Point centro;
	
	/**
	 * Costruisce un oggetto di tipo Cerchio
	 * @param centro centro del cerchio
	 * @param raggio raggio del cerchio
	 * @param r tonalita' di rosso
	 * @param g tonalita' di verde
	 * @param b tonalita' di blu
	 * @param spessore spessore del pennino
	 */
	
	public Cerchio(Point centro,double raggio,double r,double g,double b,int spessore){
		super(r,g,b,spessore);
		this.raggio=raggio;
		this.centro=centro;
	}
	
	/**
	 * Disegna il cerchio sul plotter in ingresso.
	 * @param tavola plotter su cui va disegnato il cerchio.
	 */ 
	
	public void disegna(Plotter tavola){
		//Calcola le coordinate del centro
		double x0=centro.getX();
		double y0=centro.getY();
		//Sposta il pennino nel punto corrispondente ad alfa=0
		tavola.up();
		Point start=new Point(x0+raggio*Math.cos(0),y0+raggio*Math.sin(0));
		tavola.moveTo(start);
		tavola.down();
		//Calcola i punti al variare di alfa (precisione 0.01) e li collega 
		for (double alfa=0;alfa<=2*Math.PI;alfa+=0.01){
			Point punto=new Point(x0+raggio*Math.cos(alfa),y0+raggio*Math.sin(alfa));
			tavola.moveTo(punto);
		}
		//Colleaga l'ultimo punto con il primo.
		tavola.moveTo(start);
		//Sposta il pennino al centro
		tavola.up();
		tavola.moveTo(centro);
	}
		
}
	