package es04;

/**
 * Questa classe rappresenta una prenotazione,
 * caratterizzata dal nome dello spettacolo e numero di posti prenotati.
 * Una prenotazione e' caratterizzata in modo univoco
 * da un identificatore (una stringa), generato automaticamente
 * quando un oggetto della classe viene instanziato.
 */
public class BookingNote {

    private int[] seats;
    private String id;
    private String spectacleName;

    /**
     * Crea una prenotazione
     * contenente informazioni sul nome dello spettacolo
     * e i posti prenotati
     *
     * @param name e' il nome dello spettacolo a cui si riferisce la prenotazione
     * @param seats e' l'elenco dei posti prenotati
     */
    public BookingNote(String name, int[] seats){
        this.seats = seats;
        this.spectacleName=name;
        this.id = spectacleName+"-"+seats.length+"-"+System.currentTimeMillis();
    }


    /**
     * Restituisce l'identificatore che caratterizza univocamente
     * la prenotazione.
     *
     * L'identificatore e' del tipo
     * NOMESPETTACOLO-NUMERO POSTI-NUMEROUNIVOCO
      *
     * @return stringa che rappresenta l'identificatore univoco della prenotazione
     */
    public String getId(){
        return id;
    }

    /**
     * Restituisce il nome dello spettacolo a cui fa riferimento la prenotazione
     * @return Nome dello spettacolo
     */
    public String getName(){
        return spectacleName;
    }


    /**
     * Restituisce l'elenco dei posti prenotati nella prenotazione
     *
     * @return elenco dei posti prenotati
     */
    public int[] getSeats(){
        return seats;
    }

    /**
     * Testa l'uguaglianza di prenotazioni.
     *
     * Due prenotazioni sono uguali se hanno
     * il medesimo identificatore.
     *
     * @param obj prenotazione da testare
     * @return true se sono uguali
     */
    public boolean equals(Object obj){
        return id == ((BookingNote)obj).id;
    }


    /**
     * Restituisce la rappresentazione testuale di una prenotazione,
     * che coincide con il suo identificatore univoco .
     *
     * Tale rappresentazione e' del tipo
     * NOMESPETTACOLO-NUMERO_POSTI-NUMEROUNIVOCO
     *
     * @return stringa che rappresenta univocamente la prenotazione
     */
    public String toString(){
        return id;
    }

}
