package es04;

import java.util.Vector;

/**
 * Questa classe rappresenta un prototipo di biglietteria elettronica. Implementa 
 * le interfacce TicketService e SpectacleService.
 * @author Andrea Urbini
 */

public class Service implements TicketService,SpectacleService{
	
	/** Lista degli spettacoli in programmazione.*/
	
	protected Vector spectaclesList;
	
	/**
	 * Costruisce un oggetto di tipo Service: una astrazione di biglietteria.
	 */ 
	
	public Service(){
		spectaclesList=new Vector();
	}
	
	/**
	 * Registra uno spettacolo. Viene creato un nuovo oggetto Spectacle definito 
	 * dalla stringa name e inserito nel vettore spectaclesList.
	 * @param name nome dello spettacolo da registrare.
	 */
	
	public void registerSpectacle(String name){
		spectaclesList.add(new Spectacle(name));
	}
	
	/**
	 * Servizio per prenotare i posti di uno spettacolo. Si specificano le 
	 * informazioni sullo spettacolo da vedere e sui i posti da prenotare; 
	 * il servizio prenota i posti e restituisce una prenotazione contenente 
	 * tali informazioni.
	 * @param spectacleName nome dello spettacolo da ricercare
	 * @param seats array contenente i posti che si intendono prenotare.
	 * @return Se l'operazione ha successo viene restituita una prenotazione, 
	 * al contrario se l'operazione non riesce viene restituito il valore null.
	 */ 
	
	public BookingNote bookTickets(String spectacleName,int[] seats){
		//Controllo che il nome dello spettacolo in input sia nella lista
		for (int i=0;i<spectaclesList.size();i++){
			Spectacle show=(Spectacle)spectaclesList.get(i);
			if (show.getName()==spectacleName){
				//ho trovato lo spettacolo
				int[] bookedSeats=show.getBookedSeats();
				//controllo che i posti da prenotare non lo siano gia'
				for (int j=0;j<seats.length;j++){
					for (int k=0;k<bookedSeats.length;k++){
						if (seats[j]==bookedSeats[k]){
							//i posti sono gia' occupati, prenotazione annullata
							return null;
						}
					}
				}
				//i posti sono ancora liberi. 
				//Concateno i due array
				int[] temp=new int[seats.length+bookedSeats.length];
				for (int j=0;j<seats.length;j++){
					temp[j]=seats[j];
				}
				for (int j=0;j<bookedSeats.length;j++){
					temp[j+seats.length]=bookedSeats[j];
				}
				//aggiorno l'array dei posti prenotati
				show.setBookedSeats(temp);
				//viene aggiornata la lista spettacoli
				spectaclesList.add(i,show);
				//viene creata la ricevuta
				return new BookingNote(spectacleName,seats);
			}
		}
		//non e' stato trovato lo spettacolo nella lista. Prenotazione annullata.
		return null;
	}
	
	/**
	 * Servizio per comprare i biglietti, per i quali deve esser stata fatta 
	 * una prenotazione con l'apposito servizio bookTickets.
	 * @param note ricevuta di prenotazione
	 * @return true se l'acquisto e' andato a buon fine, altrimenti false.
	 */ 
				 
	public boolean buyTickets(BookingNote note){
		//Controllo che il nome dello spettacolo nella ricevuta sia nella lista
		for (int i=0;i<spectaclesList.size();i++){
			Spectacle show=(Spectacle)spectaclesList.get(i);
			if (show.getName()==note.getName()){
				//ho trovato lo spettacolo
				int[] seats=note.getSeats();
				int[] boughtSeats=show.getBoughtSeats();
				//controllo che i posti da acquistare non  siano gia' stati acquistati
				for (int j=0;j<seats.length;j++){
					for (int k=0;k<boughtSeats.length;k++){
						if (seats[j]==boughtSeats[k]){
							//i posti sono gia' acquistati, acquisto annullato
							return false;
						}
					}
				}
				//i posti non sono ancora stati acquistati. 
				//Concateno i due array
				int[] temp=new int[seats.length+boughtSeats.length];
				for (int j=0;j<seats.length;j++){
					temp[j]=seats[j];
				}
				for (int j=0;j<boughtSeats.length;j++){
					temp[j+seats.length]=boughtSeats[j];
				}
				//aggiorno l'array dei posti prenotati
				show.setBoughtSeats(temp);
				//viene aggiornata la lista spettacoli
				spectaclesList.add(i,show);
				//l'acquisto e' andato a buon fine
				return true;
			}
		}
		//non e' stato trovato lo spettacolo nella lista. Acquisto annullato.
		return false;
	}

}