package es04;

import java.io.*;

/**
 * Questa classe rappresenta un prototipo di biglietteria elettronica. Implementa 
 * le interfacce TicketService e SpectacleService. Questa classe estende la 
 * classe Service modificando i suoi metodi: viene aggiunta la funzione di 
 * scrittura su file di testo.
 * @author Andrea Urbini
 */

public class Service2 extends Service{
	
	/**
	 * Costruisce un oggetto di tipo Service2: una astrazione di biglietteria.
	 */ 
	
	public Service2(){
		super();
	}
	
	/**
	 * Servizio per prenotare i posti di uno spettacolo. Estende il metodo della 
	 * classe Service. Se viene creata una prenotazione viene creato un file 
	 * service.txt contenente l'informazione sulla prenotazione.  
	 * @param spectacleName nome dello spettacolo da ricercare
	 * @param seats array contenente i posti che si intendono prenotare.
	 * @return Se l'operazione ha successo viene restituita una prenotazione, 
	 * al contrario se l'operazione non riesce viene restituito il valore null.
	 */ 
	
	public BookingNote bookTickets(String spectacleName,int[] seats){
		//Crea la prenotazione richiamendo il metodo della superclasse 
		BookingNote note=super.bookTickets(spectacleName,seats);
		//Se la prenotazione fallisce ritorna null
		if (note==null){
			return null;
		}else{
			/*Se la prenotazione avviene creo una stringa con l'informazione e
			 *la passo al metodo makeFile.*/
			String messaggio="Sono stati prenotati "+seats.length+" posti per lo spettacolo "+spectacleName+". Prenotazone: "+note.getId();
			makeFile(messaggio);
			return note;
		}
	}
	
	/**
	 * Servizio per comprare i biglietti, per i quali deve esser stata fatta 
	 * una prenotazione con l'apposito servizio bookTickets. Se l'acquisto e' 
	 * fattibile viene creato un file service.txt contenente l'informazione 
	 * sull'acquisto.
	 * @param note ricevuta di prenotazione
	 * @return true se l'acquisto e' andato a buon fine, altrimenti false.
	 */ 
	
	public boolean buyTickets(BookingNote note){
		if (super.buyTickets(note)){
			/*Se l'acquisto avviene creo una stringa con l'informazione e la
			 *passo al metodo makeFile.*/
			String messaggio="Sono stati acquistati i biglietti relativi alla prenotazione: "+note.getId();
			makeFile(messaggio);
			return true;
		}
		return false;
	}
	
	/**
	 * Scrive il file service.txt sull'hardDisk contenente le stringhe messaggio.
	 * @param messaggio stringa contenente le informazioni da salvare sul file
	 */  
			
	private void makeFile(String messaggio){
		FileWriter file=null;
		try{
			//Crea un oggetto di tipo FileWriter; e' abilitata la funzione append 
			file=new FileWriter("service.txt",true);
		}catch(IOException e1){
			//Se si verifica una eccezione allora stampo un messaggio di errore
			System.out.println("Impossibile aprire file");
			System.exit(1);
		}
		try{
			//scrive sul file la stringa messaggio
			file.write("\n"+messaggio+"\n");
			//Chiude lo stream
			file.close();
		}catch(IOException e2){
			//Se si verifica una eccezione allora stampo un messaggio di errore
			System.out.println("Errore di output");
			System.exit(2);
		}
	}
}
		 		