package es04;

/**
 * Questo componente software testa il comportamento
 * del servizio di prenotazione e acquisto di biglietti
 * per spettacoli teatrali
 */
public class TestService{
    public static void main(String args[]){

        // creazione di un servizio per gestione biglietti
        Service service = new Service();


        // interfaccia utilizzata dagli utenti
        TicketService user = service;

        // interfaccia usata dai gestori del teatro
        SpectacleService owner = service;

        // il gestore registra due spettacoli
        owner.registerSpectacle("Storie di ordinaria follia");
        owner.registerSpectacle("Waiting for Godot");


        // ora iniziano i test lato utente

        /*
         * tentiamo di prenotare 4 posti (di numeri 1, 4, 5, 10)
         * per lo spettacolo "West-side Story"
         *
         * siccome lo spettacolo non e' disponibile, la prenotazione deve fallire
         */
        BookingNote note = user.bookTickets("West-side Story",new int[]{1,4,5,10});
        if (note==null){
            System.out.println("Prenotazione per lo spettacolo 'West-Side Story' non riuscita");
        } else {
            System.out.println("Prenotazione riuscita: "+note);
        }


        /*
         * tentiamo di prenotare 3 posti (di numeri 1, 4, 5)
         * per lo spettacolo "Storie di ordinaria follia"
         *
         * siccome lo spettacolo e' disponibile e i posti non sono
         * ancora stati ancora occupati, la prenotazione
         * deve riuscire e quindi note2 deve contenere una
         * prenotazione valida
         */
        BookingNote note2 = user.bookTickets("Storie di ordinaria follia",new int[]{1,4,5});
        if (note2==null){
            System.out.println("Prenotazione non riuscita");
        } else {
            System.out.println("Prenotazione riuscita: "+note2);
        }

        /*
         * tentiamo di prenotare 2 posti (di numeri 5 e 6)
         * per lo spettacolo "Storie di ordinaria follia"
         *
         * siccome lo spettacolo e' disponibile MA uno dei posti e'
         * occupato, la prenotazione deve fallire
         */
        BookingNote note3 = user.bookTickets("Storie di ordinaria follia",new int[]{5,6});
        if (note3==null){
            System.out.println("Prenotazione 'Storie di ordinaria follia' (posti 5 e 6) non riuscita");
        } else {
            System.out.println("Prenotazione riuscita: "+note3);
        }

        /*
         * tentiamo di prenotare un posto (numero 20)
         * per lo spettacolo "Storie di ordinaria follia"
         *
         * siccome lo spettacolo e' disponibile e il posto e' libero
         * la prenotazione deve riuscire e quindi note4 deve contenere una nota
         * di prenotazione valida
         */
        BookingNote note4 = user.bookTickets("Storie di ordinaria follia",new int[]{20});
        if (note4==null){
            System.out.println("Prenotazione non riuscita");
        } else {
            System.out.println("Prenotazione riuscita: "+note4);
        }


        // ora testiamo l'acquisto del biglietti, date le prenotazioni

        /*
         * tentiamo di acquistare i biglietti con la prenotazione note2
         * L'acquisto deve riuscire (siccome note2 deve contenere
         * dalla fase precedente una  prenotazione valida)
         */
        boolean esito = user.buyTickets(note2);
        if (esito){
            System.out.println("Acquisto ticket "+note2+" avvenuto.");
        } else {
            System.out.println("Acquisto ticket "+note2 +" fallito.");
        }

        /*
         * tentiamo di acquistare i biglietti con la prenotazione note4
         * L'acquisto deve riuscire (siccome note4 deve contenere
         * dalla fase precedente una prenotazione valida)
         */
        esito = user.buyTickets(note4);
        if (esito){
            System.out.println("Acquisto ticket "+note4+" avvenuto.");
        } else {
            System.out.println("Acquisto ticket "+note4 +" fallito.");
        }

        /*
         * tentiamo di ri-acquistare i biglietti con la prenotazione note2
         * L'acquisto deve fallire, siccome i biglietti sono
         * gia' stati acquistati con la medesima nota
         */
        esito = user.buyTickets(note2);
        if (esito){
            System.out.println("Acquisto ticket "+note2+" avvenuto.");
        } else {
            System.out.println("Acquisto ticket "+note2 +" fallito.");
    	}
   	
    }

}
