package es05;

import java.awt.*;
import java.awt.event.*;
import java.awt.Color;
import javax.swing.*;
import java.util.Vector;

/**
 * Questa classe estende la classe JPanel definendo un nuovo panello che 
 * corrisponde alla parte centrale della finestra di ACMEPaint. 
 * La classe implementa anche l'interfaccia MouseListener quindi si rende 
 * ascoltatrice degli eventi generati dal mouse; implementa l'interfaccia 
 * ActionListener ascoltando gli eventi provenienti dagli altri due pannelli.
 * La classe, rapresentando il pannello su cui si disegna, contiene anche i 
 * metodi per spostare il cursore e disegnare linee.  
 * @author Andrea Urbini
 */

public class CenterPanel extends JPanel implements MouseListener,ActionListener {
	
	private int xMax,yMax;
	private int grid=5;
	private boolean disegna=false; 
	private Color colore=Color.black;
	private int y=140;
	private int x=200;
	private Vector listaLinee;
	private Line linea;
	private int startX,startY;
	
	/**
	 * Crea un oggetto di tipo CenterPanel. Un pannello di questo tipo genera 
	 * eventi di tipo MouseEvent che vengono ascoltati dal pannello stesso.
	 */
	
	public CenterPanel(){
		super();
		addMouseListener(this);
		setBackground(Color.white);
		listaLinee=new Vector();	
	}
	
	/**
	 * In questo metodo sono contenute le operazioni di disegno.
	 */
		
	public void paintComponent(Graphics g){
		super.paintComponent(g);
		xMax=getWidth();
		yMax=getHeight();
		//disegno le linee presenti nel vettore
		for (int i=0;i<listaLinee.size();i++){
			Line temp=(Line)listaLinee.get(i);
			g.setColor(temp.getColor());
			g.drawLine(temp.getX1(),temp.getY1(),temp.getX2(),temp.getY2());
		}
		//se esiste, disegno la variablie linea, e se viene premuto "stop" la 
		//linea viene memorizzata nel vettore
		if (linea!=null){
			g.setColor(linea.getColor());
			g.drawLine(linea.getX1(),linea.getY1(),linea.getX2(),linea.getY2());
		}
		//disegno la crocetta 	
		g.setColor(colore);
		g.drawLine(x,y+5,x,y-5);
		g.drawLine(x+5,y,x-5,y);
	}
	
	/**Con questo metodo si sposta la crocetta in alto*/
	
	public void moveUp(){
		if(y-grid>=0){
			y-=grid;
			//se  stato premuto "Start" viene disegnata una linea tra il punto corrente e il punto precedente 
			if (disegna){
				linea=new Line(startX,startY,x,y,colore);	
			}
			repaint();
		}
	}
	
	/**Con questo metodo si sposta la crocetta in basso*/
	
	public void moveDown(){
		if(y+grid<=yMax){
			y+=grid;
			//se  stato premuto "Start" viene disegnata una linea tra il punto corrente e il punto precedente 
			if (disegna){
				linea=new Line(startX,startY,x,y,colore);	
			}
			repaint();
		}
	}
	
	/**Con questo metodo si sposta la crocetta verso sinistra*/
	
	public void moveLeft(){
		if(x-grid>=0){
			x-=grid;
			//se  stato premuto "Start" viene disegnata una linea tra il punto corrente e il punto precedente 
			if (disegna){
				linea=new Line(startX,startY,x,y,colore);	
			}
			repaint();
		}
	}
	
	/**Con questo metodo si sposta la crocetta verso destra*/
	
	public void moveRight(){
		if(x+grid<=xMax){
			x+=grid;
			//se  stato premuto "Start" viene disegnata una linea tra il punto corrente e il punto precedente 
			if (disegna){
				linea=new Line(startX,startY,x,y,colore);	
			}
			repaint();
		}
	}
	
	/**
	 * Questo metodo ascolta eventi di tipo ActionEvent provenienti da JButton, 
	 * JTextField, JComboBox.
	 * @param e evento da ascoltare
	 */
	
	public void actionPerformed(ActionEvent e){
		//recupero il nome dell'oggetto che ha generato l'evento
		String name=e.getActionCommand();
		if (name.equals("Quit")){
			System.exit(0);
		}else if (name.equals("U")){
			moveUp();
		}else if (name.equals("D")){
			moveDown();
		}else if (name.equals("L")){
			moveLeft();
		}else if (name.equals("R")){
			moveRight();
		}else if (name.equals("Start")){
			//recupero l'oggetto che ha generato l'evento
			JButton button=(JButton)e.getSource();
			//cambio il testo del bottone
			button.setText("Stop");
			disegna=true;
			//fisso le coordinate da cui partira' la linea.
			startX=x;
			startY=y;
		}else if (name.equals("Stop")){
			//recupero l'oggetto che ha generato l'evento
			JButton button=(JButton)e.getSource();
			//cambio il testo del bottone
			button.setText("Start");
			disegna=false;
			//aggiungo l'oggetto assegnato a linea al vettore
			listaLinee.add(linea);
		}else if (name.equals("comboBoxChanged")){
			//recupero l'oggetto che ha generato l'evento
			JComboBox colorList=(JComboBox)e.getSource();
			//recupero il colore scelto dall'utente, e' in formato stringa, lo devo convertire
			String scelta=(String)colorList.getSelectedItem();
			if (scelta.equals("Nero"))colore=Color.black;
			if (scelta.equals("Rosso"))colore=Color.red;
			if (scelta.equals("Giallo"))colore=Color.yellow;
			if (scelta.equals("Verde"))colore=Color.green;
			if (scelta.equals("Blu"))colore=Color.blue;
			repaint();
		}else{
			//recupero l'oggetto che ha generato l'evento
			JTextField text=(JTextField)e.getSource();
			//recupero il valore della griglia
			grid=Integer.parseInt(text.getText());
			//approssimo la x ai valori della griglia
			if (x%grid!=0){
				x=(x/grid)*grid;
			}
			//approssimo la y ai valori della griglia
			if (y%grid!=0){
				y=(y/grid)*grid;
			}
			repaint();
		}
	}
	
	/**
	 * Ascoltatore del mouse. Assegna alla variabili x,y le coordinate su cui il 
	 * mouse ha cliccato.
	 * @param e evento generato dal mouse
	 */
	
	public void mouseClicked(MouseEvent e){
		x=e.getX();
		y=e.getY();
		//approssimo la x ai valori della griglia
		if (x%grid!=0){
			x=(x/grid)*grid;
		}
		//approssimo la y ai valori della griglia
		if (y%grid!=0){
			y=(y/grid)*grid;
		}
		//se  stato premuto "Start" viene disegnata una linea tra il punto corrente e il punto precedente 
		if (disegna){
			linea=new Line(startX,startY,x,y,colore);	
		}
		repaint();
	}
	
	/**metodo inutilizzato*/
	public void mousePressed(MouseEvent e){}
	/**metodo inutilizzato*/
	public void mouseReleased(MouseEvent e){}
	/**metodo inutilizzato*/
	public void mouseEntered(MouseEvent e){}
	/**metodo inutilizzato*/
	public void mouseExited(MouseEvent e){}
}